@@ -7,8 +7,7 @@ import time
 import struct
 import re
 import copy
-
-
+from collections import deque
 
 from .globaling import *
 from .odicting import odict
@@ -358,54 +357,64 @@ class Mark(object):
 
 
 class Share(object):
-    """Shared item in data store
+    """
+    Shared item in data store
 
-        so it functions somewhat like a dictionary defines:
-        __setitem__
-        __getitem__
-        __delitem__
-        __contains__
-        __iter__
-        __len__
-        clear()
-        items()
-        iteritems()
-        iterkeys()
-        itervalues()
-        keys()
-        values()
+    so it functions somewhat like a dictionary defines:
+    __setitem__
+    __getitem__
+    __delitem__
+    __contains__
+    __iter__
+    __len__
+    clear()
+    items()
+    iteritems()
+    iterkeys()
+    itervalues()
+    keys()
+    values()
 
-        instance attributes:
-            .name = holds unique store path entry name of share '.' notation
-            .store = data store holding share
-            .stamp = time stamp of this share
+    instance attributes:
+        .name = holds unique store path entry name of share '.' notation
+        .store = data store holding share
+        .stamp = time stamp of this share
+        .deck = Deck instance for this share
 
-            ._owner used by owner property
-            ._data used by data property and also by private accessor methods
-            ._truth used by truth property
-            ._unit used by unit property
+        ._owner used by owner property
+        ._data used by data property and also by private accessor methods
+        ._truth used by truth property
+        ._unit used by unit property
 
 
-        properties are used so that time stamps etc are updated properly for logging
-
-        properties (properites are stored in class):
-            .owner property holds a reference to owner of share (writer)
-            .data property holds data record
-                one time stamp applies to the whole data structure
-            .value property manages default single field value in data
-            .truth property holds the confidence of the value/data. may be None, True, False, [0.0, 1.0]
-                truth should not be updated unless value/data is, they are coupled
-                thus log if changed on truth also uses last value
-            .unit property hold units for fields
+    properties are used so that time stamps etc are updated properly for logging
 
+    properties (properites are stored in class):
+        .owner property holds a reference to owner of share (writer)
+        .data property holds data record
+            one time stamp applies to the whole data structure
+        .value property manages default single field value in data
+        .truth property holds the confidence of the value/data. may be None, True, False, [0.0, 1.0]
+            truth should not be updated unless value/data is, they are coupled
+            thus log if changed on truth also uses last value
+        .unit property hold units for fields
 
     """
-    def __init__(self, name = '',store = None,
-                 value = None, data = None, truth = None, stamp = None,
-                 unit = None, owner = None,  **kwa):
-        """Initialize instance
+    def __init__(self,
+                 name='',
+                 store=None,
+                 value=None,
+                 data=None,
+                 truth=None,
+                 stamp=None,
+                 unit=None,
+                 owner=None,
+                 deck=None,
+                 **kwa):
+        """
+        Initialize instance
 
-           Parameters:
+        Parameters:
            name = path name of share in store if created by store
            store = shared data store
            value = value of data field 'value' if any
@@ -416,12 +425,13 @@ class Share(object):
            owner = owner framework for this share
         """
 
-        self._data = Data() #new data object
+        self._data = Data()  # new data object
         self._truth = None
         self._unit = None
-
         self._owner = None
+
         self.stamp = None
+        self.deck = Deck()
 
         if not isinstance(name,str): #name must be string
             name = ''
@@ -450,6 +460,8 @@ class Share(object):
             self.changUnit(**unit)
         if owner is not None:
             self.owner = owner
+        if deck is not None and isinstance(deck, Deck):
+            self.deck = deck
 
         self.marks = odict() #odict of marks
 
@@ -539,48 +551,65 @@ class Share(object):
         """  """
         return [self[key] for key in self.keys()]
 
-    # store management
-    def changeStore(self, store = None):
+    def changeStore(self, store = None):  # store management
         """Replace .store """
         if store is not None:
             if  not isinstance(store, Store):
                 raise ValueError("Not store %s" % store)
         self.store = store
 
-    #property owner
-    def getOwner(self):
+    def stampNow(self):
+        """Force time stamp of this share to store.stamp
+           This is useful when share field is a collection that is modified
+           in place, i.e. can't use update since update copies
+           so stampNow updates the stamp.
+        """
+        self.stamp = self.store.stamp
+        return self.stamp
+
+    def push(self,  elem):
+        """
+        Convenience method to push to .deck
+        """
+        self.deck.push(elem)
+
+    def pull(self):
+        """
+        Convenience method to pull from .deck
+        """
+        return self.deck.pull()
+
+    # properties
+    def getOwner(self):  # owner property
         """getter for  owner property """
         return self._owner
 
-    def setOwner(self, owner):
+    def setOwner(self, owner):  # owner property
         """setter for owner property """
         self._owner = owner
-
     owner = property(fget = getOwner, fset = setOwner, doc = "Owner of share")
 
-    def stampNow(self):
-        """Force time stamp this share to store.stamp
-           This is useful when share field is a collection that is modified
-           in place, i.e. can't use update since update copies
-           so stampNow update the stamp.
-        """
-        self.stamp = self.store.stamp
-
-        return self.stamp
-
-    #property truth
-    def getTruth(self):
+    def getTruth(self):  # truth property
         """getter for truth property """
         return self._truth
 
-    def setTruth(self, truth):
+    def setTruth(self, truth):  # truth property
         """setter for value property """
         self._truth = truth
-
     truth = property(fget = getTruth, fset = setTruth, doc = "Truth for this share")
 
-    #property value
-    def getValue(self):
+    def getUnit(self):   # unit property
+        """getter for unit property """
+        return self._unit
+
+    def setUnit(self, unit): # unit property
+        """setter for unit property """
+        if not isinstance(unit, Data):
+            raise ValueError("Not Data object %s" % unit)
+        self._unit = unit
+    unit = property(fget = getUnit, fset = setUnit, doc = "Unit(s) for this share")
+
+    def getValue(self):  # value property
         """getter for value property
            returns none if no field in data of name 'value'
         """
@@ -589,25 +618,22 @@ class Share(object):
         except AttributeError:
             return None
 
-    def setValue(self, value):
+    def setValue(self, value):  # value property
         """setter for value property """
         setattr(self._data, 'value', value)
         self.stamp = self.store.stamp
-
     value = property(fget = getValue, fset = setValue, doc = "Value for this share")
 
-    #property data
-    def getData(self):
+    def getData(self):  # data property
         """getter for data property """
         return self._data
 
-    def setData(self, data):
+    def setData(self, data):  # data property
         """setter for data property """
         if not isinstance(data, Data):
             raise ValueError("Not Data object %s" % data)
         self._data = data
         self.stamp = self.store.stamp
-
     data = property(fget = getData, fset = setData, doc = "Data for this share")
 
     def change(self, *pa, **kwa):
@@ -623,7 +649,6 @@ class Share(object):
                     setattr(self._data, k, v)
         for k,v in kwa.items():
             setattr(self._data, k, v)
-
         return self
 
     def update(self, *pa, **kwa):
@@ -634,7 +659,6 @@ class Share(object):
         self.change(*pa, **kwa)
         #self.stamp = None #update stamp with default
         self.stamp = self.store.stamp
-
         return self
 
     def create(self, *pa, **kwa):
@@ -661,7 +685,6 @@ class Share(object):
         if update:
             #self.stamp = None #update stamp with default
             self.stamp = self.store.stamp
-
         return self
 
     def fetch(self, field, default = None):
@@ -675,19 +698,6 @@ class Share(object):
         """
         return self._data.__dict__.copy()
 
-    #property unit
-    def getUnit(self):
-        """getter for unit property """
-        return self._unit
-
-    def setUnit(self, unit):
-        """setter for unit property """
-        if not isinstance(unit, Data):
-            raise ValueError("Not Data object %s" % unit)
-        self._unit = unit
-
-    unit = property(fget = getUnit, fset = setUnit, doc = "Unit(s) for this share")
-
     def changeUnit(self, *pa, **kwa):
         """update unit from kw """
         if self.unit is None:
@@ -746,12 +756,6 @@ class Share(object):
             msg += "{0} = (1)".format(key, value)
         console.terse("{0}\n".format(msg))
 
-
-#import optimize
-
-
-#from types import ListType, TupleType
-
 class Data(object):
     """Data class
 
@@ -814,5 +818,74 @@ class Data(object):
         else: #pass on to superclass
             super(Data,self).__setattr__(key,value)
 
-#optimize.bind_all(Data)
 
+class Deck(deque):
+    """
+    Extends deque to support deque access convenience methods .push and .pull
+    to remove confusion  about which side of the deque to use (left or right).
+
+    Extends deque to support deque access convenience methods .gulp and .spew
+    to enable a different pattern for access. .gulp does not allow  a value
+    of None to be added to the Deck so retrieval with .spew can be done without
+    checking for empty. .spew returns None when empty
+
+    To determine if deck or deque is empty use
+       if d:
+
+
+    Inherited methods from deque:
+    .append(x)             = add x to right side of deque
+    .appendleft(x)         = add x to left side of deque
+    .clear()               = clear all items from deque leaving it a length 0
+    .count(x)              = count the number of deque elements equal to x.
+    .extend(iterable)      = append elements of iterable to right side
+    .extendleft(iterable)  = append elemets of iterable to left side
+                             (this reverses iterable)
+    .pop()                 = remove and return element from right side
+                              if empty then raise IndexError
+    .popleft()             = remove and return element from left side
+                              if empty then raise IndexError
+    .remove(x)             = remove first occurence of x left to right
+                              if not found raise ValueError
+    .rotate(n)             = rotate n steps to right if neg rotate to left
+
+    Built in methods supported:
+    len(d)
+    reversed(d)
+    copy.copy(d)
+    copy.deepcopy(d)
+    subscripts d[0] d[-1]
+
+    Attributes:
+    .maxlen  = maximum size of Deck or None if unbounded
+
+    Local methods:
+    .push(x)   = add x to the right side of deque (alias of append)
+    .pull(x)   = remove and return element from left side of deque (alias of popleft)
+
+    .gulp(x)    = If not None, add x to right side of deque, Otherwise ignore
+    .spew()     = remove and return element from left side of deque,
+                 If empty return None
+
+
+    """
+    push = deque.append  # alias
+    pull = deque.popleft  # alias
+
+    def gulp(self, elem):
+        """
+        If not None, add elem to right side of deque, Otherwise ignore
+        """
+        if elem is not None:
+            self.append(elem)
+
+    def spew(self):
+        """
+        Remove and return elem from left side of deque,
+        If empty return None
+        """
+        try:
+            elem = self.popleft()
+        except IndexError:
+            elem = None
+        return elem
